#!/usr/bin/env lua

Usage = [[

synopsis:
  Run a dropwhile iterator.
  Iterate over a table of integers.
  Drop (ignore) integers until a predicate fails, then process
    all the remaining integers.
  For each item that is processed
    print the iteger, the accumulated sum of integers, and
    the accumulated sum of squares of integers.
  The iterator function passes state to next iteration in an
    argument (a table), It does not use a closure.

  A simple test:
      >
      > iter = require 'Iterators.drop_while'
      > iter.run({3, 6, 9, 2, 4, 5, 7})
      pos     count   item    sum     sos
      4       1       2       2       4
      5       2       4       6       20
      6       3       5       11      45
      7       4       7       18      94
      >

  Funcion `make_dropwhile_iter` takes a predicate function and an integer
    array as arguments.  It returns an iterator function and a state (table).
    The iterator function can be called as follows:
        iterator_fn(state_tbl)
    Example:
        >
        > iter = require 'Iterators.drop_while'
        > iter_fn, state_tbl = iter.make_dropwhile_iter(p1, {4, 8, 3, 6})
        > iter_fn(state_tbl)
        3       1       3       3       9
        > iter_fn(state_tbl)
        4       2       6       9       45
        > iter_fn(state_tbl)
        nil
        >
    The state can be reset and reused by setting various attributes in it.  E.g.
        >
        > state_tbl.pos = 0
        > state_tbl.sum = 0
        > state_tbl.searching = true
        > iter_fn(state_tbl)
        3       7       3       3       179
        > iter_fn(state_tbl)
        4       8       6       9       215
        > iter_fn(state_tbl)
        nil
        >

command line usage:
  $ LUACLI=1 lua filter_false.lua <integer1> <integer1> ...

example:
  $ LUACLI=1 ./filter_false.lua 44 66 77 99 88 22
]]

local M = {}

Data = {2, 5, 3, 4}

function M.make_dropwhile_iter(predicate, data)
  data = data or Data
  local state_tbl = {
    pos = 0,
    count = 0,
    sum = 0,
    sumofsquares = 0,
    predicate = predicate,
    data = data,
    searching = true
  }
  local function iter(state)
    local int_item
    state.pos = state.pos + 1
    if state.searching then
      while true do
        if state.pos > #state.data then
          return nil
        end
        int_item = tonumber(state.data[state.pos])
        if state.predicate(int_item) then
          --print('yes', int_item)
          state.pos = state.pos + 1
        else
          --print('no', int_item)
          state.searching = false
          break
        end
      end
    end
    if state.pos > #state.data then
      return nil
    end
    int_item = tonumber(state.data[state.pos])
    state.count = state.count + 1
    state.sum = state.sum + int_item
    state.sumofsquares = state.sumofsquares + (int_item * int_item)
    return state.pos, state.count, int_item, state.sum, state.sumofsquares
  end
  return iter, state_tbl
end

function M.predicate01(item)
  return item % 3 == 0
end

function M.test(predicate, data)
  print('pos', 'count', 'item', 'sum', 'sos')
  for pos, count, item, sum, sumofsquares in M.make_dropwhile_iter(predicate, data) do
    print(pos, count, item, sum, sumofsquares)
  end
end

function M.run(args)
  if #args == 1 and (args[1] == '-h' or args[1] == '--help') then
    print(Usage)
    os.exit()
  elseif #args < 1 then
    data = nil
  else
    data = {}
    for _key, value in ipairs(args) do table.insert(data, tonumber(value)) end
  end
  M.test(M.predicate01, data)
end

if os.getenv('LUACLI') ~= nil then
  M.run(arg)
else
  return M
end
