#!/usr/bin/env lua

Usage = [[

synopsis:
  Run a filterfalse iterator.
  Iterate over a table of integers.
  For each item that satisfies the predicate,
    print the iteger, the accumulated sum of integers, and
    the accumulated sum of squares of integers.
  Passes state to each iteration in an argument (a table),
  Does not use a closure.

  A simple test:
			>
			> iter = require 'Iterators.filter_false'
			> iter.run({11, 22, 33, 44, 55})
			no      11
			yes     22
			2       1       22      22      484
			no      33
			yes     44
			4       2       44      66      2420
			no      55
			>

  Funcion `make_filterfalse_iter` takes a predicate function and an integer
    array as arguments.  It returns an iterator function and a state (table).
    The iterator function can be called as follows:
        iterator_fn(state_tbl)
    Example:
        >
        > iter_fn, state_tbl = iter.make_filterfalse_iter(iter.predicate01, {144,133, 155, 144, 166, 155})
        > iter_fn(state_tbl)
        yes     144
        1       1       144     144     20736
        > iter_fn(state_tbl)
        no      133
        no      155
        yes     144
        4       2       144     288     41472
        >
    The state can be reset and reused by setting various attributes in it.  E.g.
        state_tbl['pos'] = 0
        state_tbl['sum'] = 0

command line usage:
  $ lua filter_false.lua <int1> <int1> ...

example:
  $ LUACLI=1 ./filter_false.lua 44 66 77 99 88 22
]]

local M = {}

Data = {2, 5, 3, 4}

function M.make_filterfalse_iter(predicate, data)
  data = data or Data
  local state_tbl = {
    pos = 0,
    count = 0,
    sum = 0,
    sumofsquares = 0,
    predicate = predicate,
    data = data
  }
  local function iter(state)
    local int_item = tonumber(state.data[state.pos])
    while true do
      state.pos = state.pos + 1
      if state.pos > #state.data then
        return nil
      end
      int_item = tonumber(state.data[state.pos])
      if state.predicate(int_item) then
        print('yes', int_item)
        break
      else
        print('no', int_item)
      end
    end
    state.count = state.count + 1
    state.sum = state.sum + int_item
    state.sumofsquares = state.sumofsquares + (int_item * int_item)
    return state.pos, state.count, int_item, state.sum, state.sumofsquares
  end
  return iter, state_tbl
end

function M.predicate01(item)
  return item % 2 == 0
end

function M.test(predicate, data)
  limit = limit or 10
  for pos, count, item, sum, sumofsquares in M.make_filterfalse_iter(predicate, data) do
    print(pos, count, item, sum, sumofsquares)
  end
end

function M.run(args)
  if #args == 1 and (args[1] == '-h' or args[1] == '--help') then
    print(Usage)
    os.exit()
  elseif #args < 1 then
    data = nil
  else
    data = {}
    for _key, value in ipairs(args) do table.insert(data, tonumber(value)) end
  end
  M.test(M.predicate01, data)
end

if os.getenv('LUACLI') ~= nil then
  M.run(arg)
else
  return M
end
