#!/usr/bin/env lua

Usage = [[

synopsis:
  Run a pairwise iterator.
  Iterate over a table of integers.
  Create an array of arrays (2-tuples) of successive overlapping
    pairs taken from the input.
  The iterator function passes state to next iteration in an
    argument (a table), It does not use a closure.

  A simple test:
      >
      > iter = require 'Iterators.pairwise'
      > iter.run({3, 6, 9, 2, 4, 5, 7})
      pair:   3       6
      pair:   6       9
      pair:   9       2
      pair:   2       4
      pair:   4       5
      pair:   5       7
      >

  Funcion `make_pairwise_iter` takes an integer
    array as its argument.  It returns an iterator function and a state (table).
    The iterator function can be called as follows:
        iterator_fn(state_tbl)
    Example:
				>
				> iter = require 'Iterators.pairwise'
				> iter_fn, state_tbl = iter.make_pairwise_iter({6, 5, 4, 3, 2})
				> iter_fn(state_tbl)
				table: 0x56484217c2f0
				> iter_fn(state_tbl)
				table: 0x56484217c660
				> iter_fn(state_tbl)
				table: 0x56484217c980
				> iter_fn(state_tbl)
				table: 0x56484217ccb0
				> iter_fn(state_tbl)
				nil
				> state_tbl.pairs
				table: 0x56484217c0f0
				> utils.show_table(state_tbl.pairs)
				1. --------------------
						key: 1  value: 6
						key: 2  value: 5
				2. --------------------
						key: 1  value: 5
						key: 2  value: 4
				3. --------------------
						key: 1  value: 4
						key: 2  value: 3
				4. --------------------
						key: 1  value: 3
						key: 2  value: 2
				>

    The state can be reset and reused by setting various attributes in it.  E.g.
        >
        > state_tbl.pos = 0
        > state_tbl.pairs = {}
        >
        > iter_fn(state_tbl)
        table: 0x56484217f4a0
        > iter_fn(state_tbl)
        table: 0x56484217f8b0
        > iter_fn(state_tbl)
        table: 0x56484217fc60
        > iter_fn(state_tbl)
        table: 0x56484215eca0
        > iter_fn(state_tbl)
        nil
        > utils.show_table(state_tbl.pairs)
        1. --------------------
            key: 1  value: 6
            key: 2  value: 5
        2. --------------------
            key: 1  value: 5
            key: 2  value: 4
        3. --------------------
            key: 1  value: 4
            key: 2  value: 3
        4. --------------------
            key: 1  value: 3
            key: 2  value: 2
        >

command line usage:
  $ LUACLI=1 lua pairs.lua <integer1> <integer1> ...

example:
  $ LUACLI=1 ./pairs.lua 44 66 77 99 88 22
]]

local M = {}

Data = {2, 5, 3, 4}

function M.make_pairwise_iter(data)
  data = data or Data
  local state_tbl = {
    pos = 0,
    data = data,
    pairs = {},
  }
  local function iter(state)
    local pair
    state.pos = state.pos + 1
    if state.pos > #state.data then
      return nil
    end
    if state.pos < #state.data then
      pair = {state.data[state.pos], state.data[state.pos + 1]}
      table.insert(state.pairs, pair)
      return pair
    else
      return nil
    end
  end
  return iter, state_tbl
end

function M.test(data)
  local iter, state_tbl
  iter, state_tbl = M.make_pairwise_iter(data)
  for pair in iter, state_tbl do
  end
  local pairs = state_tbl.pairs
  for idx, value in ipairs(pairs) do
    print('pair:', value[1], value[2])
  end
end

function M.run(args)
  local data
  if #args == 1 and (args[1] == '-h' or args[1] == '--help') then
    print(Usage)
    os.exit()
  elseif #args < 1 then
    data = nil
  else
    data = {}
    for _key, value in ipairs(args) do table.insert(data, tonumber(value)) end
  end
  M.test(data)
end

if os.getenv('LUACLI') ~= nil then
  M.run(arg)
else
  return M
end
