#!/usr/bin/env lua

local Usage = [[

synopsis:
  Run a partition iterator.
  Iterate over a table of items.
  Create an array of arrays (n-tuples) of successive non-overlapping
    tuples taken from the input.
  The iterator function passes state to next iteration in an
    argument (a table), It does not use a closure.

  A simple test:
      >
      > iter = require 'Iterators.partition'
      > iter.run({3, 11, 22, 33, 44, 55, 66, 77, 88, 99})
      tuple 1:
              11
              22
              33
      tuple 2:
              44
              55
              66
      tuple 3:
              77
              88
              99
      >

  Funcion `make_partition_iter` takes an integer array and a
    partition/tuple size as its arguments.  It returns an
    iterator function and a state (table).
    The iterator function can be called as follows:
        iterator_fn(state_tbl)
    Example:
        >
        > iter_fn, state_tbl = iter.make_partition_iter({11, 22, 33, 44, 55, 66, 77, 88, 99}, 4)
        > iter_fn(state_tbl)
        table: 0x556973186e80
        > iter_fn(state_tbl)
        table: 0x556973187260
        > iter_fn(state_tbl)
        table: 0x556973199750
        > iter_fn(state_tbl)
        table: 0x556973198da0
        > iter_fn(state_tbl)
        nil
        > state_tbl.tuples
        table: 0x5569731993d0
        > utils.show_table(state_tbl.tuples)
        1. --------------------
            key: 1  value: 11
            key: 2  value: 22
            key: 3  value: 33
            key: 4  value: 44
        2. --------------------
            key: 1  value: 55
            key: 2  value: 66
            key: 3  value: 77
            key: 4  value: 88
        3. --------------------
            key: 1  value: 99
        4. --------------------
        >

    The state can be reset and reused by setting various attributes in it.  E.g.
        >
        > state_tbl.pos = 0
        > state_tbl.tuples = {}
        > state_tbl.tuplesize = 4
        > iter_fn(state_tbl)
        table: 0x55697319cb80
        > iter_fn(state_tbl)
        table: 0x55697319d420
        > iter_fn(state_tbl)
        table: 0x55697319d7c0
        > iter_fn(state_tbl)
        table: 0x55697319db80
        > iter_fn(state_tbl)
        nil
        > utils.show_table(state_tbl.tuples)
        1. --------------------
            key: 1  value: 11
            key: 2  value: 22
            key: 3  value: 33
            key: 4  value: 44
        2. --------------------
            key: 1  value: 55
            key: 2  value: 66
            key: 3  value: 77
            key: 4  value: 88
        3. --------------------
            key: 1  value: 99
        4. --------------------
        >

command line usage:
  $ LUACLI=1 lua partition.lua <tupesize> <item> <item> ...

example:
  $ LUACLI=1 ./partition.lua 4 11 22 33 44 55 66 77 88 99
]]

local M = {}

function M.make_partition_iter(data, tuplesize)
  local state_tbl = {
    pos = 0,
    data = data,
    tuplesize = tuplesize or 2,
    tuples = {},
  }
  local function iter_fn(state)
    if state.pos >= #state.data then
      return nil
    end
    local tuple = {}
    for idx = 1, state.tuplesize do
      table.insert(tuple, state.data[state.pos + idx])
    end
    table.insert(state.tuples, tuple)
    state.pos = state.pos + state.tuplesize
    return tuple
  end
  return iter_fn, state_tbl
end

function M.test(data, tuplesize)
  local iter_fn, state_tbl
  iter_fn, state_tbl = M.make_partition_iter(data, tuplesize)
  for tuple in iter_fn, state_tbl do
    -- no-op
  end
  local tuples = state_tbl.tuples
  for idx, value in ipairs(tuples) do
    print(string.format('tuple %d:', idx))
    for i, v in ipairs(value) do print('', v) end
  end
end

function M.run(args)
  local data
  if (#args == 1 and (args[1] == '-h' or args[1] == '--help')) or
    #args < 1 then
    print(Usage)
    os.exit()
  elseif #args < 2 then
    tuplesize = tonumber(args[1])
    data = nil
  else
    tuplesize = tonumber(args[1])
    data = {}
    for idx = 2, #args do
      table.insert(data, args[idx])
    end
  end
  M.test(data, tuplesize)
end

if os.getenv('LUACLI') ~= nil then
  M.run(arg)
else
  return M
end
